class ManagedWPHostingRegenerateThumbnails_Later { 
	public $fullsizepath; 
	private $thumbslaterkey = 'mwp_thumbslater'; 
	private $batch_limit = 1; 

	public function __construct( ) { 
		global $wpdb; 
		
		if( (isset( $_REQUEST['action'] ) && 'upload-attachment' === $_REQUEST['action'] ) && ! isset( $_POST['post_id'] ) ) { 
			add_filter( 'intermediate_image_sizes_advanced', array( $this, 'intermediate_image_sizes_advanced' ), 999, 2 ); 
			add_filter( 'wp_generate_attachment_metadata', array( $this, 'wp_generate_attachment_metadata' ), 999, 2 ); 
		} else { 
			$sql = $wpdb->prepare( 'SELECT post_id FROM '. $wpdb->postmeta . ' WHERE meta_key = %s LIMIT %d', $this->thumbslaterkey, $this->batch_limit ); 

			foreach( $wpdb->get_results( $sql ) as $thumbs_later ) { 
				if( $this->init( $thumbs_later->post_id ) ) { 
					delete_post_meta( $thumbs_later->post_id, $this->thumbslaterkey ); 
				}
			}
		}
	}

	public function intermediate_image_sizes_advanced( $sizes, $metadata ) { 
		define( 'MWP_THUMBSLATER_FLAG', true ); return array( ); 
	}

	public function wp_generate_attachment_metadata ( $metadata, $attachment_id ) { 
		if( defined( 'MWP_THUMBSLATER_FLAG' ) || ( isset( $metadata['sizes'] ) && $metadata['sizes'] == array () ) ) { 
			add_post_meta( $attachment_id, $this->thumbslaterkey, true ); 
		} 
		return $metadata;
	}

	private function init( $attachment_id = 0 ) {
		if( !$attachment_id ) { 
			return;
		}
		
		$attachment = get_post( $attachment_id );
		if ( ! $attachment ) { 
			return new WP_Error( 'regenerate_thumbnails_regenerator_attachment_doesnt_exist', __( 'No attachment exists with that ID.', 'regenerate-thumbnails' ), array( 'status' => 404, ) ); 
		} 

		if ( 'attachment' !== get_post_type( $attachment ) ) { 
			return new WP_Error( 'regenerate_thumbnails_regenerator_not_attachment', __( 'This item is not an attachment.', 'regenerate-thumbnails' ), array( 'status' => 400, ) ); 
		}
		
		if ( self::is_site_icon( $attachment ) ) {
			return new WP_Error( 'regenerate_thumbnails_regenerator_is_site_icon', __( "This attachment is a site icon and therefore the thumbnails shouldn't be touched.", 'regenerate-thumbnails' ), array( 'status' => 415, 'attachment' => $attachment, ) ); 
		}
		$this->attachment = $attachment;
		return $this->regenerate( );
	}

	private function get_fullsizepath() { 
		if ( $this->fullsizepath ) { 
			return $this->fullsizepath;
		}

		$this->fullsizepath = get_attached_file( $this->attachment->ID ); 

		if ( false === $this->fullsizepath || ! file_exists( $this->fullsizepath ) ) {
			$error = new WP_Error( 'regenerate_thumbnails_regenerator_file_not_found', sprintf( __( "The fullsize image file cannot be found in your uploads directory at <code>%s</code>. Without it, new thumbnail images can't be generated.", 'regenerate-thumbnails' ), _wp_relative_upload_path( $this->fullsizepath ) ), array( 'status' => 404, 'fullsizepath' => _wp_relative_upload_path( $this->fullsizepath ), 'attachment' => $this->attachment, ) );
		
			$this->fullsizepath = $error;
		}
		return $this->fullsizepath;
	}

	private static function is_site_icon( WP_Post $attachment ) {
		return ( 'site-icon' === get_post_meta( $attachment->ID, '_wp_attachment_context', true ) ); 
	}

	private function regenerate( $args = array() ) {
		$args = wp_parse_args( $args, array( ) );
		$fullsizepath = $this->get_fullsizepath();
		if ( is_wp_error( $fullsizepath ) ) { 
			$fullsizepath->add_data( array( 'attachment' => $this->attachment ) ); 
			return $fullsizepath;
		}
		
		$old_metadata = wp_get_attachment_metadata( $this->attachment->ID ); 
		require_once( ABSPATH . 'wp-admin/includes/admin.php' ); 
		$new_metadata = wp_generate_attachment_metadata( $this->attachment->ID, $fullsizepath ); 
		wp_update_attachment_metadata( $this->attachment->ID, $new_metadata ); 
		return $new_metadata; 
	}
}

function ManagedWPHostingRegenerateThumbnails_Later() {
	$ManagedWPHostingRegenerateThumbnails_Later = new ManagedWPHostingRegenerateThumbnails_Later(); 
}
